#########################################################################
#									
# 	This R Program simulates an adaptation of the method of 
#	Conaway, Dunbar, and Peddada (2004) for use in the 
#	CNLVINB drug combination study 														
#									
#########################################################################

#install.packages("Iso")
library(Iso)

###function for stage 1 ####
CDPstage1 <- function (currlev, trueprob,scheme) #the output contains (yvec, nvec)
{	#Currlev most time starts with level 1.
	#trueprob is a vector containging the true toxicity probability for each dose.
	#possesc is a list indicating the possible escalation for each dose.  The length of this list is equal to the levels of doses.

if(scheme=="slow"){
	x0=c(5,6,sample(7:9),sample(10:13),sample(14:17),sample(18:20),sample(21:22),23)
}
if(scheme=="fast"){
	x0=c(5,7,11,15,19,21,23)
}
	nvec <-yvec <- rep(0, length(trueprob))
	i <-1

	#currlev is the matrix index for the dose in the doseind.  It is not the value of doseind.
while(i <= length(x0)){
   {	#print('stage 1')
	#print(currlev)
	#print(trueprob[currlev])

	yi <- (runif(1, min=0, max=1) <= trueprob[currlev]) #whether a DLt or not
	#print(yi)
	nvec[currlev] <- 1+nvec[currlev]

	if (yi) 
	{	yvec[currlev] <- 1 + yvec[currlev]
		break  #go to stage 2
	}


	if(!yi) 
	{	
		currlev <- x0[i+1]
			}
	}

	if (is.na(currlev)) break
	i=i+1
    }	 #end of repeat
	
    cbind(yvec, nvec)
}


##figure out the dose for next patient, used by function stage2.
sugglevfun <- function (havemin, possesc, target, userind, pi, nvec, yvec)
{		#only one has mini loss
		if (sum(havemin)==1)
		{	sugglev <- userind[havemin]
		}
		
		#more than 1 mini loss
		if (sum(havemin) > 1)
		 {	#mini loss has ties
			posslev <- userind [havemin]			
			if (all(pi[posslev] >= target)) #doses with miniloss all have pi larger than target
			{	sugglev <- min(posslev)
			}
 			else	#some dose with pi less than target

			{	posslev2 <-  posslev [pi[posslev] < target]
				sugglev <- max(posslev2)
			}
		}

		#if the sugglev less than target and some of its possible esclation haven't been tried, we need to esclate.

		len <- length(possesc[[sugglev]])
		if (len==1)	poss <- possesc[[sugglev]][[1]]
		if (len==2)	poss <- c(possesc[[sugglev]][[1]], possesc[[sugglev]][[2]])		

		if (pi[sugglev] < target && any (nvec[poss] == 0)) 
		{	posslev <- possesc[[sugglev]] #need to esclate.  
			len <- length(posslev)	#check it is 1 or 2 direction choice
		
			posslev2 <- list()
			t <- match(0, nvec[posslev[[1]]])  #find the 1st not tried dose in the 1st direction.
			if (!is.na(t))	posslev2[[1]] <- posslev[[1]][t]
		

			if (len==2)
			{	t2 <- match(0, nvec[posslev[[2]]])  #find the 1st not tried dose in the 2nd direction.
				if(!is.na(t2))
				{	if (!is.na(t))	posslev2[[2]]<- posslev[[2]][t2]
				}
				else posslev2[[2]] <- posslev[[2]][t2]

			}

			
			nesc <- length(posslev2)

			if (nesc==1) sugglev <- posslev2[[1]][1]
			
			if (nesc==2)
			{	if (is.na(posslev2[[2]])) sugglev <- posslev2[[1]][1]

				if(!is.na(posslev2[[2]])) 
				{	tmp <- runif(nesc, min=0, max=1)
					sugglev <- posslev2[[match(T,rank(tmp)== nesc)]][1]
				}
			}
		}

	sugglev
}


###function for stage 2 #####
CDPstage2 <- function (yvec, nvec, cohortsize, target, stoprule, nmax, trueprob, possesc, possord,cl)
{	

###calculate beta prior
x<-target*2
mu<-target
u<-0.95

f<-function(b){
	pbeta(x,mu*b/(1-mu),b)-u
}
b0<-uniroot(f,c(0.0001,100))$root
a0<-mu*b0/(1-mu)
avec=a0
bvec=b0
gam=0.5

	len <- length(yvec)
	userind <- seq(1, len, 1)
	mtd<-0
	a <- length(possord[1,])  #decide how many possible orders.
	study=1
	repeat 

	{	
		##count how many on each dose, if too many, then stop the trial
		if(any(nvec>=stoprule)) 	
		{	stop<- 1
			break
		}

		if (sum(nvec)>=nmax)
		{	stop<-2
			break
		}

		if(1 - pbeta(target, yvec[1] + a0, nvec[1] - yvec[1] + b0) > cl){
			stop<-3
			break
		}

		
		#get pava estimate
		x <- (yvec+avec)/(nvec+avec+bvec)
		y <- cbind(x, nvec, 1:len)

		pi.1<-rep(0, len)

	
		#print(yvec)
		#print(nvec)
		#print(round(x, 2))

		for ( m in 1:a)
		{	y2 <- y[possord[,m],]
			x2 <- y2[,1]
			wt <- y2[,2]
			pi0 <- pava(x2, w=wt)	
			tmp <- sort(y2[,3], index.return=T)
			pi.1 <- pi.1 + pi0[tmp[[2]]]
			#print(round(pi0[tmp[[2]]], 2))		

		}

		pi <- round(pi.1/a,2)  #pi is an average of estimates from the a types of possible orders.
		#print(round(pi, 2))

		if((1 - pbeta(target, yvec[5] + a0, nvec[5] - yvec[5] + b0) > cl) & (1 - pbeta(target, yvec[6] + a0, nvec[6] - yvec[6] + b0) > cl)){
			study=2
		}

		lossvec=rep(1,len)
		if(study==1){
			lossvec[5:23]=ifelse(pi[5:23] > target, (1-gam)*(pi[5:23]-target), gam*(target-pi[5:23]))
		}
		if(study==2){
			lossvec[1:4]=ifelse(pi[1:4] > target, (1-gam)*(pi[1:4]-target), gam*(target-pi[1:4]))
		}
		#lossvec	

		##calculate loss function
		#lossvec <- ifelse(pi > target, (1-gam)*(pi-target), gam*(target-pi))
		minloss <- min(lossvec)
		havemin <- lossvec==minloss
		if(study==2 & sum(nvec[1:4])==0){
			sugglev=1
		} else{
			sugglev <- sugglevfun(havemin, possesc, target, userind, pi, nvec, yvec)
		}
		suggprob <- pi[sugglev]
	
	
		#print(yvec)
		#print(nvec)
		#print(pi)
		#print(paste('suggested level', sugglev))


		##at this point, the cycle is going on and a dose has been decided for the next patient
		## generate an outcome for this new patient and add the outcome to the stop criteria
		
		currlev <- sugglev
	
		nvec[currlev] <- cohortsize+nvec[currlev]
		yi <- (runif(cohortsize, min=0, max=1) <= trueprob[currlev]) #whether a DLt or not
		yvec[currlev] <- sum(yi) + yvec[currlev]

		
	}	## end of repeat

if(stop!=3){
	#get pava estimate
	x <- (yvec+avec)/(nvec+avec+bvec)
	y <- cbind(x, nvec, 1:len)
	pi.1<-rep(0, len)



	for ( m in 1:a)
	{	y2 <- y[possord[,m],]
		x2 <- y2[,1]
		wt <- y2[,2]
		pi0 <- pava(x2, w=wt)	
		tmp <- sort(y2[,3], index.return=T)
		pi.1 <- pi.1 + pi0[tmp[[2]]]
	}
	pi <- round(pi.1/a,2)
	if((1 - pbeta(target, yvec[5] + a0, nvec[5] - yvec[5] + b0) > cl) & (1 - pbeta(target, yvec[6] + a0, nvec[6] - yvec[6] + b0) > cl)){
			study=2
		}


	##calculate loss function
	lossvec=rep(1,len)
		if(study==1){
			lossvec[5:23]=ifelse(pi[5:23] > target, (1-gam)*(pi[5:23]-target), gam*(target-pi[5:23]))
		}
		if(study==2){
			lossvec[1:4]=ifelse(pi[1:4] > target, (1-gam)*(pi[1:4]-target), gam*(target-pi[1:4]))
		}

	minloss <- min(lossvec)
	havemin <- lossvec==minloss


	#figure out the final MTD
	mtd <- sugglevfun(havemin, possesc, target, userind, pi, nvec, yvec)
		
		#print(yvec)
		#print(nvec)
		#print(pi)
		#print(mtd)
}
	
	##output the cycle results
		list(yvec, nvec, pi, mtd, stop)
		
}	##end of stage 2
		


##final simulation function###
CDPdesign <- function (cycle, target, trueprob, possesc, possord, nmax, stoprule, cohortsize, cl)

{   

    nlev=length(trueprob)
    doseind <- seq(1, nlev, 1)
    toxtotal<-ntotal<-mtd<-mtdprob<-stop<-trialsize<-dltper<-rep(0, cycle)
    npts<-ntox<-matrix(,nrow=cycle,ncol=nlev)
    for (i in 1: cycle) { 

	nvec <- rep(0, nlev) #store number of patients assigned to each dose.
	yvec <- rep(0, nlev) #store number of toxicity occurred to each dose.

	#Stage 1#
	s1out <- s1out <- CDPstage1(currlev=5, trueprob=trueprob, scheme)
	#s1out

	#stage 2#
	s2out <- CDPstage2(yvec=s1out[,1], nvec=s1out[,2],target=target,trueprob=trueprob, possesc=possesc, possord=possord, cohortsize=cohortsize, stoprule=stoprule, nmax=nmax,cl)
	#s2out
	#sum(s2out[[2]])

	mtd[i] <- s2out[[4]]
	npts[i,]<- s2out[[2]]
	ntox[i,]<- s2out[[1]]
	stop[i] <- s2out[[5]]
	trialsize[i] <- sum(s2out[[2]])
	dltper[i] <- sum(s2out[[1]])/sum(s2out[[2]])


 }													

    cat("Simulation results for Conaway, Dunbar and Peddada (2004)\n");
    cat("design targeting a DLT rate of", target,"\n\n");
    cat("True DLT probability:\n");
    cat(round(trueprob,3), sep="\t",  "\n");
    cat("MTD selection percentage:\n");
    cat(formatC(tabulate(mtd, nbins = nlev)/cycle*100, digits=2, format="f"), sep="\t",  "\n");
    cat("Avg number pts treated:\n");
    cat(formatC(colMeans(npts), digits=2, format="f"), sep="\t",   "\n");
    cat("Avg percentage of DLTs:\n");
    cat(round(mean(dltper)*100,3), sep="\t",  "\n");
    cat("Avg sample size:\n");
    cat(formatC(sum(colMeans(npts)), digits=2, format="f"), sep="\t",   "\n");
    cat("Sample size percentiles (25th,50th,75th,90th,95th):\n")
    cat(c(fivenum(trialsize)[2:4],quantile(trialsize,0.90)[[1]],quantile(trialsize,0.95)[[1]]), "\n")
    cat("Accuracy index:\n");
    cat(round(1-length(trueprob)*(sum(abs(trueprob-target)*(tabulate(mtd, nbins = nlev)/cycle))/sum(abs(trueprob-target))),4), sep="\t",  "\n");
    cat("Percent stopped for safety:\n");
    cat(formatC(100*sum(mtd==0)/cycle, digits=2, format="f"), sep="\t",   "\n");
    cat("MTD selection percentage within 5 points of target:\n");
    cat(sum((tabulate(mtd, nbins = nlev)/cycle)[which(abs(trueprob-target)<=0.051)])*100, sep="\t",  "\n");
    cat("Avg number pts treated within 5 points of target:\n");
    cat(round(sum(colMeans(npts)[which(abs(trueprob-target)<=0.071)]),1), sep="\t",   "\n");
    cat("MTD selection percentage within 7 points of target:\n");
    cat(sum((tabulate(mtd, nbins = nlev)/cycle)[which(abs(trueprob-target)<=0.051)])*100, sep="\t",  "\n");
    cat("Avg number pts treated within 7 points of target:\n");
    cat(round(sum(colMeans(npts)[which(abs(trueprob-target)<=0.071)]),1), sep="\t",   "\n");
    cat("MTD selection percentage within 10 points of target:\n");
    cat(sum((tabulate(mtd, nbins = nlev)/cycle)[which(abs(trueprob-target)<=0.101)])*100, sep="\t",  "\n");
    cat("Avg number pts treated within 10 points of target:\n");
    cat(round(sum(colMeans(npts)[which(abs(trueprob-target)<=0.101)]),1), sep="\t",   "\n");
    sink("newfile.txt", append=TRUE, split=TRUE)
} #end of CDPdesign

################################################################################
#
#	Input
#
#	cycle = number of simulations
#	target = target DLT rate defining MTD
#	trueprob = true probability of DLT at each combination
#	possesc = the possible eslcation for each combination. It is a list.
#	nmax= max number of patients
#	stoperule = max number of patients on a single dose
#	cohortsize=cohort size 
#
#	scheme = Stage 1 esclation scheme; either 'column' or 'diagonal
#		   'column' = cohorts of size 1 up the column until first DLT
#		   'diagonal' = cohorts of size 2 by zone of matrix
#
#
#	Combination labels
#	4 14 18 21 23
#	3 10 15 19 22
#	2  7 11 16 20
#	1  5  8 12 17
#		6  9 13
#
#################################################################################
target<-1/5
cycle<-1000
nmax<-60
stoprule<-12
cohortsize<-1
possesc<-list(2,3,4,4,c(7,8),c(8,9),c(10,11),c(11,12),c(12,13),c(14,15),c(15,16),c(16,17),17,18,c(18,19),c(19,20),20,
		21,c(21,22),22,23,23,23) ##need to be length 1 or 2
possord<-matrix(nrow=23,ncol=6)
possord[,1]<-c(1,2,3,4,6,5,7,8,9,13,12,11,10,14,15,16,17,20,19,18,21,22,23)  
possord[,2]<-c(1,2,3,4,5,6,9,8,7,10,11,12,13,17,16,15,14,18,19,20,22,21,23)  
possord[,3]<-c(1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23)
possord[,4]<-c(1,2,3,4,6,5,9,8,7,13,12,11,10,17,16,15,14,20,19,18,22,21,23)
possord[,5]<-c(1,2,3,4,6,9,13,5,8,12,17,7,11,16,20,10,15,19,22,14,18,21,23)
possord[,6]<-c(1,2,3,4,5,7,10,14,6,8,11,15,18,9,12,16,19,21,13,17,20,22,23)
scheme="fast"  
cl=0.7

#Gina's sceanrios
p1<-c(0.01,0.02,0.03,0.05,0.04,0.05,0.05,0.06,0.08,0.06,0.07,0.09,0.12,0.08,0.09,0.10,0.13,0.11,0.12,0.14,0.15,0.16,0.20)
p2<-c(0.01,0.02,0.03,0.05,0.05,0.07,0.06,0.08,0.10,0.07,0.09,0.11,0.13,0.10,0.11,0.12,0.14,0.15,0.15,0.15,0.20,0.20,0.30)
p3<-c(0.01,0.02,0.03,0.15,0.04,0.06,0.05,0.07,0.10,0.10,0.09,0.11,0.14,0.25,0.18,0.15,0.15,0.35,0.26,0.20,0.45,0.35,0.55)
p4<-c(0.01,0.02,0.05,0.15,0.05,0.08,0.07,0.09,0.11,0.12,0.12,0.14,0.15,0.25,0.20,0.20,0.20,0.35,0.31,0.29,0.47,0.43,0.60)
p5<-c(0.01,0.02,0.05,0.15,0.04,0.05,0.07,0.10,0.15,0.14,0.16,0.20,0.25,0.25,0.24,0.27,0.31,0.35,0.36,0.39,0.48,0.49,0.62)
p6<-c(0.25,0.29,0.33,0.37,0.29,0.29,0.33,0.33,0.34,0.37,0.37,0.38,0.39,0.41,0.41,0.42,0.43,0.45,0.46,0.47,0.50,0.51,0.55)+0.05

trueprob<-p6
set.seed(34472)
CDPdesign(cycle, target, trueprob, possesc, possord, nmax, stoprule, cohortsize,cl)


